#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
from __future__ import annotations

from typing import TYPE_CHECKING

from oc_ocdm.decorators import accepts_only
from oc_ocdm.metadata.metadata_entity import MetadataEntity
from rdflib import XSD

if TYPE_CHECKING:
    from typing import Optional, List
    from rdflib import URIRef
    from oc_ocdm.metadata.entities.distribution import Distribution


class Dataset(MetadataEntity):
    """Dataset (short: not applicable and strictly dependent on the implementation of the
       dataset infrastructure): a set of collected information about something."""

    @accepts_only('_dataset_')
    def merge(self, other: Dataset) -> None:
        """
        The merge operation allows combining two ``Dataset`` entities into a single one,
        by marking the second entity as to be deleted while also copying its data into the current
        ``Dataset``. Moreover, every triple from the containing ``MetadataSet`` referring to the second
        entity gets "redirected" to the current entity: **every other reference contained inside a
        different source (e.g. a triplestore) must be manually handled by the user!**

        In case of functional properties, values from the current entity get overwritten
        by those coming from the second entity while, in all other cases, values from the
        second entity are simply appended to those of the current entity. In this context,
        ``rdfs:label`` is considered as a functional property, while ``rdf:type`` is not.

        :param other: The entity which will be marked as to be deleted and whose properties will
         be merged into the current entity.
        :type other: Dataset
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        super(Dataset, self).merge(other)

        title: Optional[str] = other.get_title()
        if title is not None:
            self.has_title(title)

        description: Optional[str] = other.get_description()
        if description is not None:
            self.has_description(description)

        pub_date: Optional[str] = other.get_publication_date()
        if pub_date is not None:
            self.has_publication_date(pub_date)

        mod_date: Optional[str] = other.get_modification_date()
        if mod_date is not None:
            self.has_modification_date(mod_date)

        keywords_list: List[str] = other.get_keywords()
        for cur_keyword in keywords_list:
            self.has_keyword(cur_keyword)

        subjects_list: List[URIRef] = other.get_subjects()
        for cur_subject in subjects_list:
            self.has_subject(cur_subject)

        landing_page: Optional[URIRef] = other.get_landing_page()
        if landing_page is not None:
            self.has_landing_page(landing_page)

        sub_datasets_list: List[Dataset] = other.get_sub_datasets()
        for cur_sub_dataset in sub_datasets_list:
            self.has_sub_dataset(cur_sub_dataset)

        sparql_endpoint: Optional[URIRef] = other.get_sparql_endpoint()
        if sparql_endpoint is not None:
            self.has_sparql_endpoint(sparql_endpoint)

        distributions_list: List[Distribution] = other.get_distributions()
        for cur_distribution in distributions_list:
            self.has_distribution(cur_distribution)

    # HAS TITLE
    def get_title(self) -> Optional[str]:
        """
        Getter method corresponding to the ``dcterms:title`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        return self._get_literal(MetadataEntity.iri_title)

    @accepts_only('literal')
    def has_title(self, string: str) -> None:
        """
        Setter method corresponding to the ``dcterms:title`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `The title of the dataset.`

        :param string: The value that will be set as the object of the property related to this method
        :type string: str
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.remove_title()
        self._create_literal(MetadataEntity.iri_title, string)

    def remove_title(self) -> None:
        """
        Remover method corresponding to the ``dcterms:title`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, MetadataEntity.iri_title, None))

    # HAS DESCRIPTION
    def get_description(self) -> Optional[str]:
        """
        Getter method corresponding to the ``dcterms:description`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        return self._get_literal(MetadataEntity.iri_description)

    @accepts_only('literal')
    def has_description(self, string: str) -> None:
        """
        Setter method corresponding to the ``dcterms:description`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `A short textual description of the content of the dataset.`

        :param string: The value that will be set as the object of the property related to this method
        :type string: str
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.remove_description()
        self._create_literal(MetadataEntity.iri_description, string)

    def remove_description(self) -> None:
        """
        Remover method corresponding to the ``dcterms:description`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, MetadataEntity.iri_description, None))

    # HAS PUBLICATION DATE
    def get_publication_date(self) -> Optional[str]:
        """
        Getter method corresponding to the ``dcterms:issued`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        return self._get_literal(MetadataEntity.iri_issued)

    @accepts_only('literal')
    def has_publication_date(self, string: str) -> None:
        """
        Setter method corresponding to the ``dcterms:issued`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `The date of first publication of the dataset.`

        :param string: The value that will be set as the object of the property related to this method. **It must
          be a string compliant with the** ``xsd:dateTime`` **datatype.**
        :type string: str
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.remove_publication_date()
        self._create_literal(MetadataEntity.iri_issued, string, XSD.dateTime, False)

    def remove_publication_date(self) -> None:
        """
        Remover method corresponding to the ``dcterms:issued`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, MetadataEntity.iri_issued, None))

    # HAS MODIFICATION DATE
    def get_modification_date(self) -> Optional[str]:
        """
        Getter method corresponding to the ``dcterms:modified`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        return self._get_literal(MetadataEntity.iri_modified)

    @accepts_only('literal')
    def has_modification_date(self, string: str) -> None:
        """
        Setter method corresponding to the ``dcterms:modified`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `The date on which the dataset has been modified.`

        :param string: The value that will be set as the object of the property related to this method. **It must
          be a string compliant with the** ``xsd:dateTime`` **datatype.**
        :type string: str
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.remove_modification_date()
        self._create_literal(MetadataEntity.iri_modified, string, XSD.dateTime, False)

    def remove_modification_date(self) -> None:
        """
        Remover method corresponding to the ``dcterms:modified`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, MetadataEntity.iri_modified, None))

    # HAS KEYWORD
    def get_keywords(self) -> List[str]:
        """
        Getter method corresponding to the ``dcat:keyword`` RDF predicate.

        :return: A list containing the requested values if found, None otherwise
        """
        return self._get_multiple_literals(MetadataEntity.iri_keyword)

    @accepts_only('literal')
    def has_keyword(self, string: str) -> None:
        """
        Setter method corresponding to the ``dcat:keyword`` RDF predicate.

        `A keyword or phrase describing the content of the dataset.`

        :param string: The value that will be set as the object of the property related to this method
        :type string: str
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self._create_literal(MetadataEntity.iri_keyword, string)

    @accepts_only('literal')
    def remove_keyword(self, string: str = None) -> None:
        """
        Remover method corresponding to the ``dcat:keyword`` RDF predicate.

        **WARNING: this is a non-functional property, hence, if the parameter
        is None, any existing value will be removed!**

        :param string: If not None, the specific object value that will be removed from the property
         related to this method (defaults to None)
        :type string: str
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        if string is not None:
            self.g.remove((self.res, MetadataEntity.iri_keyword, string))
        else:
            self.g.remove((self.res, MetadataEntity.iri_keyword, None))

    # HAS SUBJECT
    def get_subjects(self) -> List[URIRef]:
        """
        Getter method corresponding to the ``dcat:theme`` RDF predicate.

        :return: A list containing the requested values if found, None otherwise
        """
        uri_list: List[URIRef] = self._get_multiple_uri_references(MetadataEntity.iri_subject)
        return uri_list

    @accepts_only('thing')
    def has_subject(self, thing_res: URIRef) -> None:
        """
        Setter method corresponding to the ``dcat:theme`` RDF predicate.

        `A concept describing the primary subject of the dataset.`

        :param thing_res: The value that will be set as the object of the property related to this method
        :type thing_res: URIRef
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.g.add((self.res, MetadataEntity.iri_subject, thing_res))

    @accepts_only('thing')
    def remove_subject(self, thing_res: URIRef = None) -> None:
        """
        Remover method corresponding to the ``dcat:theme`` RDF predicate.

        **WARNING: this is a non-functional property, hence, if the parameter
        is None, any existing value will be removed!**

        :param thing_res: If not None, the specific object value that will be removed from the property
         related to this method (defaults to None)
        :type thing_res: URIRef
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        if thing_res is not None:
            self.g.remove((self.res, MetadataEntity.iri_subject, thing_res))
        else:
            self.g.remove((self.res, MetadataEntity.iri_subject, None))

    # HAS LANDING PAGE
    def get_landing_page(self) -> Optional[URIRef]:
        """
        Getter method corresponding to the ``dcat:landingPage`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        return self._get_literal(MetadataEntity.iri_landing_page)

    @accepts_only('thing')
    def has_landing_page(self, thing_res: URIRef) -> None:
        """
        Setter method corresponding to the ``dcat:landingPage`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `An HTML page (indicated by its URL) representing a browsable page for the dataset.`

        :param thing_res: The value that will be set as the object of the property related to this method
        :type thing_res: URIRef
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.remove_landing_page()
        self.g.add((self.res, MetadataEntity.iri_landing_page, thing_res))

    def remove_landing_page(self) -> None:
        """
        Remover method corresponding to the ``dcat:landingPage`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, MetadataEntity.iri_landing_page, None))

    # HAS SUB-DATASET
    def get_sub_datasets(self) -> List[Dataset]:
        """
        Getter method corresponding to the ``void:subset`` RDF predicate.

        :return: A list containing the requested values if found, None otherwise
        """
        uri_list: List[URIRef] = self._get_multiple_uri_references(MetadataEntity.iri_subset, '_dataset_')
        result: List[Dataset] = []
        for uri in uri_list:
            result.append(self.m_set.add_dataset(self.resp_agent, self.source, uri))
        return result

    @accepts_only('_dataset_')
    def has_sub_dataset(self, obj: Dataset) -> None:
        """
        Setter method corresponding to the ``void:subset`` RDF predicate.

        `A link to a subset of the present dataset.`

        :param obj: The value that will be set as the object of the property related to this method
        :type obj: Dataset
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.g.add((self.res, MetadataEntity.iri_subset, obj.res))

    @accepts_only('_dataset_')
    def remove_sub_dataset(self, dataset_res: Dataset = None) -> None:
        """
        Remover method corresponding to the ``void:subset`` RDF predicate.

        **WARNING: this is a non-functional property, hence, if the parameter
        is None, any existing value will be removed!**

        :param dataset_res: If not None, the specific object value that will be removed from the property
         related to this method (defaults to None)
        :type dataset_res: Dataset
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        if dataset_res is not None:
            self.g.remove((self.res, MetadataEntity.iri_subset, dataset_res.res))
        else:
            self.g.remove((self.res, MetadataEntity.iri_subset, None))

    # HAS SPARQL ENDPOINT
    def get_sparql_endpoint(self) -> Optional[URIRef]:
        """
        Getter method corresponding to the ``void:sparqlEndpoint`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        uri: Optional[URIRef] = self._get_uri_reference(MetadataEntity.iri_sparql_endpoint)
        return uri

    @accepts_only('thing')
    def has_sparql_endpoint(self, thing_res: URIRef) -> None:
        """
        Setter method corresponding to the ``void:sparqlEndpoint`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `The link to the SPARQL endpoint for querying the dataset.`

        :param thing_res: The value that will be set as the object of the property related to this method
        :type thing_res: URIRef
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.remove_sparql_endpoint()
        self.g.add((self.res, MetadataEntity.iri_sparql_endpoint, thing_res))

    def remove_sparql_endpoint(self) -> None:
        """
        Remover method corresponding to the ``void:sparqlEndpoint`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, MetadataEntity.iri_sparql_endpoint, None))

    # HAS DISTRIBUTION (Distribution)
    def get_distributions(self) -> List[Distribution]:
        """
        Getter method corresponding to the ``dcat:distribution`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        uri_list: List[URIRef] = self._get_multiple_uri_references(MetadataEntity.iri_distribution, 'di')
        result: List[Distribution] = []
        for uri in uri_list:
            result.append(self.m_set.add_di(self.resp_agent, self.source, uri))
        return result

    @accepts_only('di')
    def has_distribution(self, obj: Distribution) -> None:
        """
        Setter method corresponding to the ``dcat:distribution`` RDF predicate.

        `A distribution of the dataset.`

        :param obj: The value that will be set as the object of the property related to this method
        :type obj: Distribution
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.g.add((self.res, MetadataEntity.iri_distribution, obj.res))

    @accepts_only('di')
    def remove_distribution(self, di_res: Distribution = None) -> None:
        """
        Remover method corresponding to the ``dcat:distribution`` RDF predicate.

        **WARNING: this is a non-functional property, hence, if the parameter
        is None, any existing value will be removed!**

        :param di_res: If not None, the specific object value that will be removed from the property
         related to this method (defaults to None)
        :type di_res: Distribution
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        if di_res is not None:
            self.g.remove((self.res, MetadataEntity.iri_distribution, di_res.res))
        else:
            self.g.remove((self.res, MetadataEntity.iri_distribution, None))



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.

from oc_ocdm.counter_handler.counter_handler import CounterHandler
from oc_ocdm.counter_handler.filesystem_counter_handler import FilesystemCounterHandler
from oc_ocdm.counter_handler.in_memory_counter_handler import InMemoryCounterHandler



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
import os
import unittest

from oc_ocdm.counter_handler.filesystem_counter_handler import FilesystemCounterHandler


class TestFilesystemCounterHandler(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        cls.info_dir = './info_dir/'
        cls.counter_handler = FilesystemCounterHandler(cls.info_dir)

        cls.file_path = cls.info_dir + 'test_file.txt'
        if not os.path.exists(os.path.dirname(cls.file_path)):
            os.makedirs(os.path.dirname(cls.file_path))

        if not os.path.isfile(cls.file_path):
            with open(cls.file_path, 'wb') as file:
                first_line = cls.counter_handler._trailing_char * (cls.counter_handler._initial_line_len - 1) + '\n'
                file.write(first_line.encode('ascii'))

    def setUp(self):
        # Reset test file content:
        with open(self.file_path, 'wb') as file:
            first_line = self.counter_handler._trailing_char * (self.counter_handler._initial_line_len - 1) + '\n'
            file.write(first_line.encode('ascii'))

    def test_get_line_len(self):
        with open(self.file_path, 'rb') as test_file:
            line_len = self.counter_handler._get_line_len(test_file)
            self.assertIsNotNone(line_len)
            self.assertEqual(line_len, self.counter_handler._initial_line_len)
            self.assertEqual(test_file.tell(), 0)

    def test_increase_line_len(self):
        increment = 1
        result = self.counter_handler._increase_line_len(self.file_path,
                                                         self.counter_handler._initial_line_len + increment)
        self.assertIsNone(result)
        with open(self.file_path, 'rt', encoding='ascii') as test_file:
            for line in test_file:
                self.assertEqual(len(line), self.counter_handler._initial_line_len + increment)

        self.assertRaises(ValueError, self.counter_handler._increase_line_len, self.file_path, -1)
        self.assertRaises(ValueError, self.counter_handler._increase_line_len, self.file_path,
                          self.counter_handler._initial_line_len - 1)

    def test_is_a_valid_line(self):
        with self.subTest("line is 'abc \\n'"):
            line = 'abc \n'.encode('ascii')
            result = self.counter_handler._is_a_valid_line(line)
            self.assertIsNotNone(result)
            self.assertTrue(result)
        with self.subTest("line is 'a\\0c \\n'"):
            line = 'a\0c \n'.encode('ascii')
            result = self.counter_handler._is_a_valid_line(line)
            self.assertIsNotNone(result)
            self.assertFalse(result)
        with self.subTest("line is 'abc'"):
            line = 'abc'.encode('ascii')
            result = self.counter_handler._is_a_valid_line(line)
            self.assertIsNotNone(result)
            self.assertFalse(result)
        with self.subTest("line is 'a\\0c'"):
            line = 'a\\0c'.encode('ascii')
            result = self.counter_handler._is_a_valid_line(line)
            self.assertIsNotNone(result)
            self.assertFalse(result)

    def test_fix_previous_lines(self):
        with open(self.file_path, 'wb') as test_file:
            num_lines = 10
            for i in range(0, num_lines):
                line = '\0' * self.counter_handler._initial_line_len
                test_file.write(line.encode('ascii'))
            last_line = '1'.ljust(self.counter_handler._initial_line_len - 1, self.counter_handler._trailing_char) + '\n'
            test_file.write(last_line.encode('ascii'))

        with open(self.file_path, 'r+b') as test_file:
            test_file.seek(self.counter_handler._initial_line_len * num_lines)
            result = self.counter_handler._fix_previous_lines(test_file, self.counter_handler._initial_line_len)
            self.assertIsNone(result)

        with open(self.file_path, 'rt', encoding='ascii') as test_file:
            count = 0
            for line in test_file:
                count += 1
                if count >= num_lines:
                    break
                self.assertTrue(self.counter_handler._is_a_valid_line(line.encode('ascii')))

    def test_set_number(self):
        number = 18
        with open(self.file_path, 'r+b') as test_file:
            num_of_line = 35
            test_file.seek(self.counter_handler._initial_line_len * (num_of_line - 1))
            line = str(number).ljust(self.counter_handler._initial_line_len - 1, self.counter_handler._trailing_char) + '\n'
            test_file.write(line.encode('ascii'))

        new_number = 205
        result = self.counter_handler._set_number(new_number, self.file_path, num_of_line)
        self.assertIsNone(result)
        with open(self.file_path, 'rt', encoding='ascii') as test_file:
            count = 0
            for line in test_file:
                count += 1
                if count >= num_of_line:
                    self.assertEqual(int(line), new_number)
                    break
                self.assertTrue(self.counter_handler._is_a_valid_line(line.encode('ascii')))

        self.assertRaises(ValueError, self.counter_handler._set_number, -1, self.file_path, 1)
        self.assertRaises(ValueError, self.counter_handler._set_number, 1, self.file_path, -1)

    def test_read_number(self):
        number = 18
        with open(self.file_path, 'r+b') as test_file:
            num_of_line = 35
            test_file.seek(self.counter_handler._initial_line_len * (num_of_line - 1))
            line = str(number).ljust(self.counter_handler._initial_line_len - 1, self.counter_handler._trailing_char) + '\n'
            test_file.write(line.encode('ascii'))

        read_number, line_len = self.counter_handler._read_number(self.file_path, num_of_line)
        self.assertIsNotNone(read_number)
        self.assertIsNotNone(line_len)
        self.assertEqual(read_number, number)
        self.assertEqual(line_len, self.counter_handler._initial_line_len)

        self.assertRaises(ValueError, self.counter_handler._read_number, self.file_path, -1)

    def test_add_number(self):
        number = 18
        with open(self.file_path, 'r+b') as test_file:
            num_of_line = 35
            test_file.seek(self.counter_handler._initial_line_len * (num_of_line - 1))
            line = str(number).ljust(self.counter_handler._initial_line_len - 1, self.counter_handler._trailing_char) + '\n'
            test_file.write(line.encode('ascii'))

        read_number = self.counter_handler._add_number(self.file_path, num_of_line)
        self.assertIsNotNone(read_number)
        self.assertEqual(read_number, number + 1)
        with open(self.file_path, 'rt', encoding='ascii') as test_file:
            count = 0
            for line in test_file:
                count += 1
                if count >= num_of_line:
                    break
                self.assertTrue(self.counter_handler._is_a_valid_line(line.encode('ascii')))

        self.assertRaises(ValueError, self.counter_handler._add_number, self.file_path, -1)

    def test_read_metadata_counter(self):
        dataset_name: str = "http://dataset/"
        self.assertRaises(ValueError, self.counter_handler.read_metadata_counter, "xyz", dataset_name)
        self.assertRaises(ValueError, self.counter_handler.read_metadata_counter, "di", None)

    def test_increment_metadata_counter(self):
        dataset_name: str = "http://dataset/"
        self.assertRaises(ValueError, self.counter_handler.increment_metadata_counter, "xyz", dataset_name)
        self.assertRaises(ValueError, self.counter_handler.increment_metadata_counter, "di", None)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
from __future__ import annotations

from typing import TYPE_CHECKING

from oc_ocdm.decorators import accepts_only

if TYPE_CHECKING:
    from typing import Optional, List
    from rdflib import URIRef
    from oc_ocdm.graph.entities.bibliographic.bibliographic_reference import BibliographicReference
    from oc_ocdm.graph.entities.bibliographic.reference_annotation import ReferenceAnnotation
from oc_ocdm.graph.graph_entity import GraphEntity
from oc_ocdm.graph.entities.bibliographic_entity import BibliographicEntity


class ReferencePointer(BibliographicEntity):
    """Reference pointer (long: in-text reference pointer; short: rp): a textual device (e.g.
       '[1]'), denoting a single bibliographic reference, that is embedded in the text of a
       document within the context of a particular sentence or text chunk. A bibliographic
       reference can be denoted in the text by one or more in-text reference pointers."""

    @accepts_only('rp')
    def merge(self, other: ReferencePointer) -> None:
        """
        The merge operation allows combining two ``ReferencePointer`` entities into a single one,
        by marking the second entity as to be deleted while also copying its data into the current
        ``ReferencePointer``. Moreover, every triple from the containing ``GraphSet`` referring to the second
        entity gets "redirected" to the current entity: **every other reference contained inside a
        different source (e.g. a triplestore) must be manually handled by the user!**

        In case of functional properties, values from the current entity get overwritten
        by those coming from the second entity while, in all other cases, values from the
        second entity are simply appended to those of the current entity. In this context,
        ``rdfs:label`` is considered as a functional property, while ``rdf:type`` is not.

        :param other: The entity which will be marked as to be deleted and whose properties will
         be merged into the current entity.
        :type other: ReferencePointer
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        super(ReferencePointer, self).merge(other)

        content: Optional[str] = other.get_content()
        if content is not None:
            self.has_content(content)

        next_rp: Optional[ReferencePointer] = other.get_next_rp()
        if next_rp is not None:
            self.has_next_rp(next_rp)

        denoted_be: Optional[BibliographicReference] = other.get_denoted_be()
        if denoted_be is not None:
            self.denotes_be(denoted_be)

        an_list: List[ReferenceAnnotation] = other.get_annotations()
        for cur_an in an_list:
            self.has_annotation(cur_an)

    # HAS REFERENCE POINTER TEXT
    def get_content(self) -> Optional[str]:
        """
        Getter method corresponding to the ``c4o:hasContent`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        return self._get_literal(GraphEntity.iri_has_content)

    @accepts_only('literal')
    def has_content(self, string: str) -> None:
        """
        Setter method corresponding to the ``c4o:hasContent`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `The literal text of the textual device forming an in-text reference pointer and denoting
        a single bibliographic reference (e.g. “[1]”).`

        :param string: The value that will be set as the object of the property related to this method
        :type string: str
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.remove_content()
        self._create_literal(GraphEntity.iri_has_content, string)

    def remove_content(self) -> None:
        """
        Remover method corresponding to the ``c4o:hasContent`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, GraphEntity.iri_has_content, None))

    # HAS NEXT (ReferencePointer)
    def get_next_rp(self) -> Optional[ReferencePointer]:
        """
        Getter method corresponding to the ``oco:hasNext`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        uri: Optional[URIRef] = self._get_uri_reference(GraphEntity.iri_has_next, 'rp')
        if uri is not None:
            return self.g_set.add_rp(self.resp_agent, self.source, uri)

    @accepts_only('rp')
    def has_next_rp(self, rp_res: ReferencePointer) -> None:
        """
        Setter method corresponding to the ``oco:hasNext`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `The following in-text reference pointer, when included within a single in-text reference
        pointer list.`

        :param rp_res: The value that will be set as the object of the property related to this method
        :type rp_res: ReferencePointer
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.remove_next_rp()
        self.g.add((self.res, GraphEntity.iri_has_next, rp_res.res))

    def remove_next_rp(self) -> None:
        """
        Remover method corresponding to the ``oco:hasNext`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, GraphEntity.iri_has_next, None))

    # DENOTES (BibliographicReference)
    def get_denoted_be(self) -> Optional[BibliographicReference]:
        """
        Getter method corresponding to the ``c4o:denotes`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        uri: Optional[URIRef] = self._get_uri_reference(GraphEntity.iri_denotes, 'be')
        if uri is not None:
            return self.g_set.add_be(self.resp_agent, self.source, uri)

    @accepts_only('be')
    def denotes_be(self, be_res: BibliographicReference) -> None:
        """
        Setter method corresponding to the ``c4o:denotes`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `The bibliographic reference included in the list of bibliographic references, denoted by
        the in-text reference pointer.`

        :param be_res: The value that will be set as the object of the property related to this method
        :type be_res: BibliographicReference
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.remove_denoted_be()
        self.g.add((self.res, GraphEntity.iri_denotes, be_res.res))

    def remove_denoted_be(self) -> None:
        """
        Remover method corresponding to the ``c4o:denotes`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, GraphEntity.iri_denotes, None))

    # HAS ANNOTATION (ReferenceAnnotation)
    def get_annotations(self) -> List[ReferenceAnnotation]:
        """
        Getter method corresponding to the ``oco:hasAnnotation`` RDF predicate.

        :return: A list containing the requested values if found, None otherwise
        """
        uri_list: List[URIRef] = self._get_multiple_uri_references(GraphEntity.iri_has_annotation, 'an')
        result: List[ReferenceAnnotation] = []
        for uri in uri_list:
            result.append(self.g_set.add_an(self.resp_agent, self.source, uri))
        return result

    @accepts_only('an')
    def has_annotation(self, an_res: ReferenceAnnotation) -> None:
        """
        Setter method corresponding to the ``oco:hasAnnotation`` RDF predicate.

        `An annotation characterizing the citation to which the in-text reference pointer relates
        in terms of its citation function (the reason for that citation) specific to the textual
        location of that in-text reference pointer within the citing entity.`

        :param an_res: The value that will be set as the object of the property related to this method
        :type an_res: ReferenceAnnotation
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.g.add((self.res, GraphEntity.iri_has_annotation, an_res.res))

    @accepts_only('an')
    def remove_annotation(self, an_res: ReferenceAnnotation = None) -> None:
        """
        Remover method corresponding to the ``oco:hasAnnotation`` RDF predicate.

        **WARNING: this is a non-functional property, hence, if the parameter
        is None, any existing value will be removed!**

        :param an_res: If not None, the specific object value that will be removed from the property
         related to this method (defaults to None)
        :type an_res: ReferenceAnnotation
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        if an_res is not None:
            self.g.remove((self.res, GraphEntity.iri_has_annotation, an_res.res))
        else:
            self.g.remove((self.res, GraphEntity.iri_has_annotation, None))



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
import unittest

from rdflib import URIRef, Literal, XSD, RDF

from oc_ocdm.graph.graph_entity import GraphEntity
from oc_ocdm.graph.graph_set import GraphSet


class TestBibliographicResource(unittest.TestCase):
    resp_agent = 'http://resp_agent.test/'

    @classmethod
    def setUpClass(cls) -> None:
        cls.graph_set = GraphSet("http://test/", "./info_dir/", "", False)

    def setUp(self):
        self.br1 = self.graph_set.add_br(self.resp_agent)
        self.br2 = self.graph_set.add_br(self.resp_agent)
        self.re = self.graph_set.add_re(self.resp_agent)
        self.be = self.graph_set.add_be(self.resp_agent)
        self.de = self.graph_set.add_de(self.resp_agent)
        self.ar = self.graph_set.add_ar(self.resp_agent)

    def test_has_title(self):
        title = "Resource"
        result = self.br1.has_title(title)
        self.assertIsNone(result)

        triple = self.br1.res, GraphEntity.iri_title, Literal(title)
        self.assertIn(triple, self.br1.g)

    def test_has_subtitle(self):
        subtitle = "Resource"
        result = self.br1.has_subtitle(subtitle)
        self.assertIsNone(result)

        triple = self.br1.res, GraphEntity.iri_has_subtitle, Literal(subtitle)
        self.assertIn(triple, self.br1.g)

    def test_is_part_of(self):
        result = self.br1.is_part_of(self.br2)
        self.assertIsNone(result)

        triple = self.br1.res, GraphEntity.iri_part_of, self.br2.res
        self.assertIn(triple, self.br1.g)

    def test_has_citation(self):
        result = self.br1.has_citation(self.br2)
        self.assertIsNone(result)

        triple = self.br1.res, GraphEntity.iri_cites, self.br2.res
        self.assertIn(triple, self.br1.g)

    def test_has_pub_date(self):
        with self.subTest("date is '2020-05-25'"):
            string = "2020-05-25"
            datatype = XSD.date
            result = self.br1.has_pub_date(string)
            self.assertIsNone(result)

            triple = self.br1.res, GraphEntity.iri_has_publication_date, Literal(string, datatype=datatype,
                                                                                 normalize=False)
            self.assertIn(triple, self.br1.g)
        with self.subTest("date is '2020-05'"):
            string = "2020-05"
            datatype = XSD.gYearMonth
            result = self.br1.has_pub_date(string)
            self.assertIsNone(result)

            triple = self.br1.res, GraphEntity.iri_has_publication_date, Literal(string, datatype=datatype,
                                                                                 normalize=False)
            self.assertIn(triple, self.br1.g)
        with self.subTest("date is '2020'"):
            string = "2020"
            datatype = XSD.gYear
            result = self.br1.has_pub_date(string)
            self.assertIsNone(result)

            triple = self.br1.res, GraphEntity.iri_has_publication_date, Literal(string, datatype=datatype,
                                                                                 normalize=False)
            self.assertIn(triple, self.br1.g)

    def test_has_format(self):
        result = self.br1.has_format(self.re)
        self.assertIsNone(result)

        triple = self.br1.res, GraphEntity.iri_embodiment, self.re.res
        self.assertIn(triple, self.br1.g)

    def test_create_number(self):
        number = "1234"
        result = self.br1.has_number(number)
        self.assertIsNone(result)

        triple = self.br1.res, GraphEntity.iri_has_sequence_identifier, Literal(number)
        self.assertIn(triple, self.br1.g)

    def test_has_edition(self):
        edition = "abcde"
        result = self.br1.has_edition(edition)
        self.assertIsNone(result)

        triple = self.br1.res, GraphEntity.iri_has_edition, Literal(edition)
        self.assertIn(triple, self.br1.g)

    def test_contains_in_reference_list(self):
        result = self.br1.contains_in_reference_list(self.be)
        self.assertIsNone(result)

        triple = self.br1.res, GraphEntity.iri_contains_reference, self.be.res
        self.assertIn(triple, self.br1.g)

    def test_contains_discourse_element(self):
        result = self.br1.contains_discourse_element(self.de)
        self.assertIsNone(result)

        triple = self.br1.res, GraphEntity.iri_contains_de, self.de.res
        self.assertIn(triple, self.br1.g)

    def test_has_contributor(self):
        result = self.br1.has_contributor(self.ar)
        self.assertIsNone(result)

        triple = self.br1.res, GraphEntity.iri_is_document_context_for, self.ar.res
        self.assertIn(triple, self.br1.g)

    def test_has_related_document(self):
        document = URIRef("http://test/document")
        result = self.br1.has_related_document(document)
        self.assertIsNone(result)

        triple = self.br1.res, GraphEntity.iri_relation, document
        self.assertIn(triple, self.br1.g)

    def test_create_archival_document(self):
        result = self.br1.create_archival_document()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_archival_document
        self.assertIn(triple, self.br1.g)

    def test_create_book(self):
        result = self.br1.create_book()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_book
        self.assertIn(triple, self.br1.g)

    def test_create_book_chapter(self):
        result = self.br1.create_book_chapter()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_book_chapter
        self.assertIn(triple, self.br1.g)

    def test_create_book_part(self):
        result = self.br1.create_book_part()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_part
        self.assertIn(triple, self.br1.g)

    def test_create_book_section(self):
        result = self.br1.create_book_section()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_expression_collection
        self.assertIn(triple, self.br1.g)

    def test_create_book_series(self):
        result = self.br1.create_book_series()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_book_series
        self.assertIn(triple, self.br1.g)

    def test_create_book_set(self):
        result = self.br1.create_book_set()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_book_set
        self.assertIn(triple, self.br1.g)

    def test_create_book_track(self):
        result = self.br1.create_book_track()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_expression
        self.assertIn(triple, self.br1.g)

    def test_create_component(self):
        result = self.br1.create_component()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_expression
        self.assertIn(triple, self.br1.g)

    def test_create_dataset(self):
        result = self.br1.create_dataset()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_data_file
        self.assertIn(triple, self.br1.g)

    def test_create_dissertation(self):
        result = self.br1.create_dissertation()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_thesis
        self.assertIn(triple, self.br1.g)

    def test_create_edited_book(self):
        result = self.br1.create_edited_book()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_book
        self.assertIn(triple, self.br1.g)

    def test_create_journal_article(self):
        result = self.br1.create_journal_article()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_journal_article
        self.assertIn(triple, self.br1.g)

    def test_create_issue(self):
        result = self.br1.create_issue()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_journal_issue
        self.assertIn(triple, self.br1.g)

    def test_create_volume(self):
        result = self.br1.create_volume()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_journal_volume
        self.assertIn(triple, self.br1.g)

    def test_create_journal(self):
        result = self.br1.create_journal()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_journal
        self.assertIn(triple, self.br1.g)

    def test_create_monograph(self):
        result = self.br1.create_monograph()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_book
        self.assertIn(triple, self.br1.g)

    def test_create_proceedings_article(self):
        result = self.br1.create_proceedings_article()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_proceedings_paper
        self.assertIn(triple, self.br1.g)

    def test_create_proceedings(self):
        result = self.br1.create_proceedings()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_academic_proceedings
        self.assertIn(triple, self.br1.g)

    def test_create_reference_book(self):
        result = self.br1.create_reference_book()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_reference_book
        self.assertIn(triple, self.br1.g)

    def test_create_reference_entry(self):
        result = self.br1.create_reference_entry()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_reference_entry
        self.assertIn(triple, self.br1.g)

    def test_create_report_series(self):
        result = self.br1.create_report_series()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_series
        self.assertIn(triple, self.br1.g)

    def test_create_report(self):
        result = self.br1.create_report()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_report_document
        self.assertIn(triple, self.br1.g)

    def test_create_standard_series(self):
        result = self.br1.create_standard_series()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_series
        self.assertIn(triple, self.br1.g)

    def test_create_standard(self):
        result = self.br1.create_standard()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_specification_document
        self.assertIn(triple, self.br1.g)

    def test_create_series(self):
        result = self.br1.create_series()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_series
        self.assertIn(triple, self.br1.g)

    def test_create_expression_collection(self):
        result = self.br1.create_expression_collection()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_expression_collection
        self.assertIn(triple, self.br1.g)

    def test_create_other(self):
        result = self.br1.create_other()
        self.assertIsNone(result)

        triple = self.br1.res, RDF.type, GraphEntity.iri_expression
        self.assertIn(triple, self.br1.g)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
import unittest

from rdflib import Graph, BNode

from oc_ocdm.graph.graph_set import GraphSet

from oc_ocdm.metadata.metadata_set import MetadataSet
from oc_ocdm.metadata.entities.dataset import Dataset
from oc_ocdm.metadata.entities.distribution import Distribution


class TestMetadataSet(unittest.TestCase):
    resp_agent = 'http://resp_agent.test/'

    def setUp(self):
        self.metadata_set = MetadataSet("http://test/", "./info_dir/", False)

    def test_get_entity(self):
        di = self.metadata_set.add_di("ocdmTest", self.resp_agent)
        ref = di.res
        result = self.metadata_set.get_entity(ref)
        self.assertIsNotNone(result)
        self.assertIs(result, di)

    def test_add_dataset(self):
        dataset = self.metadata_set.add_dataset("ocdmTest", self.resp_agent)

        self.assertIsNotNone(dataset)
        self.assertIsInstance(dataset, Dataset)
        self.assertIsInstance(dataset.g.identifier, BNode)

    def test_add_di(self):
        di = self.metadata_set.add_di("ocdmTest", self.resp_agent)

        self.assertIsNotNone(di)
        self.assertIsInstance(di, Distribution)
        self.assertIsInstance(di.g.identifier, BNode)

    def test_graphs(self):
        count = 10
        for i in range(count):
            self.metadata_set.add_di("ocdmTest", self.resp_agent)
        result = self.metadata_set.graphs()
        self.assertIsNotNone(result)
        self.assertEqual(len(result), count)
        for graph in result:
            self.assertIsInstance(graph, Graph)

    def test_get_graph_iri(self):
        ar = self.metadata_set.add_dataset("ocdmTest", self.resp_agent)
        iri = str(ar.g.identifier)
        result = GraphSet.get_graph_iri(ar.g)
        self.assertIsNotNone(result)
        self.assertEqual(iri, result)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
from abc import ABC, abstractmethod


class CounterHandler(ABC):
    """Abstract class representing the interface for every concrete counter handler."""

    @abstractmethod
    def set_counter(self, new_value: int, entity_short_name: str, prov_short_name: str = "",
                    identifier: int = 1) -> None:
        """
        Method signature for concrete implementations that allow setting the counter value
        of graph and provenance entities.

        :param new_value: The new counter value to be set
        :type new_value: int
        :param entity_short_name: The short name associated either to the type of the entity itself
         or, in case of a provenance entity, to the type of the relative graph entity.
        :type entity_short_name: str
        :param prov_short_name: In case of a provenance entity, the short name associated to the type
         of the entity itself. An empty string otherwise.
        :type prov_short_name: str
        :param identifier: In case of a provenance entity, the counter value that identifies the relative
          graph entity. The integer value '1' otherwise.
        :type identifier: int
        :raises NotImplementedError: always
        :return: None
        """
        raise NotImplementedError

    @abstractmethod
    def read_counter(self, entity_short_name: str, prov_short_name: str = "", identifier: int = 1) -> int:
        """
        Method signature for concrete implementations that allow reading the counter value
        of graph and provenance entities.

        :param entity_short_name: The short name associated either to the type of the entity itself
         or, in case of a provenance entity, to the type of the relative graph entity.
        :type entity_short_name: str
        :param prov_short_name: In case of a provenance entity, the short name associated to the type
         of the entity itself. An empty string otherwise.
        :type prov_short_name: str
        :param identifier: In case of a provenance entity, the counter value that identifies the relative
          graph entity. The integer value '1' otherwise.
        :type identifier: int
        :raises NotImplementedError: always
        :return: The requested counter value.
        """
        raise NotImplementedError

    @abstractmethod
    def increment_counter(self, entity_short_name: str, prov_short_name: str = "", identifier: int = 1) -> int:
        """
        Method signature for concrete implementations that allow incrementing by one unit
        the counter value of graph and provenance entities.

        :param entity_short_name: The short name associated either to the type of the entity itself
         or, in case of a provenance entity, to the type of the relative graph entity.
        :type entity_short_name: str
        :param prov_short_name: In case of a provenance entity, the short name associated to the type
         of the entity itself. An empty string otherwise.
        :type prov_short_name: str
        :param identifier: In case of a provenance entity, the counter value that identifies the relative
          graph entity. The integer value '1' otherwise.
        :type identifier: int
        :raises NotImplementedError: always
        :return: The newly-updated (already incremented) counter value.
        """
        raise NotImplementedError

    @abstractmethod
    def set_metadata_counter(self, new_value: int, entity_short_name: str, dataset_name: str) -> None:
        """
        Method signature for concrete implementations that allow setting the counter value
        of metadata entities.

        :param new_value: The new counter value to be set
        :type new_value: int
        :param entity_short_name: The short name associated either to the type of the entity itself.
        :type entity_short_name: str
        :param dataset_name: In case of a ``Dataset``, its name. Otherwise, the name of the relative dataset.
        :type dataset_name: str
        :raises NotImplementedError: always
        :return: None
        """
        raise NotImplementedError

    @abstractmethod
    def read_metadata_counter(self, entity_short_name: str, dataset_name: str) -> int:
        """
        Method signature for concrete implementations that allow reading the counter value
        of metadata entities.

        :param entity_short_name: The short name associated either to the type of the entity itself.
        :type entity_short_name: str
        :param dataset_name: In case of a ``Dataset``, its name. Otherwise, the name of the relative dataset.
        :type dataset_name: str
        :raises NotImplementedError: always
        :return: The requested counter value.
        """
        raise NotImplementedError

    @abstractmethod
    def increment_metadata_counter(self, entity_short_name: str, dataset_name: str) -> int:
        """
        Method signature for concrete implementations that allow incrementing by one unit
        the counter value of metadata entities.

        :param entity_short_name: The short name associated either to the type of the entity itself.
        :type entity_short_name: str
        :param dataset_name: In case of a ``Dataset``, its name. Otherwise, the name of the relative dataset.
        :type dataset_name: str
        :raises NotImplementedError: always
        :return: The newly-updated (already incremented) counter value.
        """
        raise NotImplementedError



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
from __future__ import annotations

from typing import TYPE_CHECKING

from oc_ocdm.decorators import accepts_only

if TYPE_CHECKING:
    from typing import List, Dict, Optional
    from rdflib import URIRef
    from oc_ocdm.graph.entities.identifier import Identifier
from oc_ocdm.graph.graph_entity import GraphEntity


class BibliographicEntity(GraphEntity):
    """The base class for each bibliographic entity of the OpenCitations DataModel (OCDM)."""

    def merge(self, other: BibliographicEntity) -> None:
        """
        **WARNING:** ``BibliographicEntity`` **is an abstract class that cannot be instantiated at runtime.
        As such, it's only possible to execute this method on entities generated from**
        ``BibliographicEntity``'s **subclasses. Please, refer to their documentation of the** `merge` **method.**

        :param other: The entity which will be marked as to be deleted and whose properties will
         be merged into the current entity.
        :type other: BibliographicEntity
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        super(BibliographicEntity, self).merge(other)

        id_list: List[Identifier] = other.get_identifiers()
        for cur_id in id_list:
            self.has_identifier(cur_id)

        # The special semantics associated to the identifiers
        # of a bibliographic entity requires them to be uniquely
        # defined based on their scheme and literal value:
        self.remove_duplicated_identifiers()

    # HAS IDENTIFIER
    def get_identifiers(self) -> List[Identifier]:
        """
        Getter method corresponding to the ``datacite:hasIdentifier`` RDF predicate.

        :return: A list containing the requested values if found, None otherwise
        """
        uri_list: List[URIRef] = self._get_multiple_uri_references(GraphEntity.iri_has_identifier, 'id')
        result: List[Identifier] = []
        for uri in uri_list:
            result.append(self.g_set.add_id(self.resp_agent, self.source, uri))
        return result

    @accepts_only('id')
    def has_identifier(self, id_res: Identifier) -> None:
        """
        Setter method corresponding to the ``datacite:hasIdentifier`` RDF predicate.

        `In addition to the internal dataset identifier assigned to the entity upon initial
        curation (format: [entity short name]/[local identifier]), other external third-party
        identifiers can be specified through this attribute (e.g. DOI, ORCID, PubMedID).`

        :param id_res: The value that will be set as the object of the property related to this method
        :type id_res: Identifier
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.g.add((self.res, GraphEntity.iri_has_identifier, id_res.res))

    @accepts_only('id')
    def remove_identifier(self, id_res: Identifier = None) -> None:
        """
        Remover method corresponding to the ``datacite:hasIdentifier`` RDF predicate.

        **WARNING: this is a non-functional property, hence, if the parameter
        is None, any existing value will be removed!**

        :param id_res: If not None, the specific object value that will be removed from the property
         related to this method (defaults to None)
        :type id_res: Identifier
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        if id_res is not None:
            self.g.remove((self.res, GraphEntity.iri_has_identifier, id_res.res))
        else:
            self.g.remove((self.res, GraphEntity.iri_has_identifier, None))

    def remove_duplicated_identifiers(self) -> None:
        """
        Utility function that automatically scans the list of Identifier entities associated to the
        current bibliographic entity (through the ``datacite:hasIdentifier`` RDF predicate) and it removes
        duplicated entries.

        Two distinct ``Identifier`` entities are considered the same if they share both
        the scheme (``datacite:usesIdentifierScheme``) and the literal value (``literal:hasLiteralValue``).

        :return: None
        """

        # Identifiers should be merged based on the
        # correspondence between both their scheme and literal value!
        id_list: List[Identifier] = self.get_identifiers()
        # We remove every identifier from 'self': only unique ones
        # will be re-associated with 'self'.
        self.remove_identifier()

        # We use a nested dictionary which associates the 'schema-literal_value'
        # pair to the corresponding identifier object
        # (ex. id_dict[ISSN][1234-5678] <- base_iri:id/34).
        id_dict: Dict[URIRef, Dict[str, Identifier]] = {}
        for identifier in id_list:
            schema: Optional[URIRef] = identifier.get_scheme()
            literal_value: Optional[str] = identifier.get_literal_value()
            if schema is not None and literal_value is not None:
                if schema not in id_dict:
                    id_dict[schema] = {literal_value: identifier}
                    self.has_identifier(identifier)  # the Identifier is kept!
                else:
                    if literal_value not in id_dict[schema]:
                        id_dict[schema][literal_value] = identifier
                        self.has_identifier(identifier)  # the Identifier is kept!
                    else:
                        id_to_be_kept: Identifier = id_dict[schema][literal_value]
                        id_to_be_kept.merge(identifier)  # the Identifier is dropped!



import os
import unittest

from pyshacl import validate
from rdflib import ConjunctiveGraph, Graph

from oc_ocdm.graph.graph_set import GraphSet
from oc_ocdm.reader import Reader


class TestShacle(unittest.TestCase):
    def test_validate(self):
      data_graph = ConjunctiveGraph()
      sg = Graph()
      data_graph.parse(source=os.path.join('oc_ocdm', 'test', 'resources', 'data.json'), format='json-ld')
      sg.parse(source=os.path.join('oc_ocdm', 'resources', 'shacle.ttl'), format='text/turtle')
      r = validate(data_graph,
        shacl_graph=sg,
        ont_graph=None,
        inference=None,
        abort_on_first=False,
        allow_infos=False,
        allow_warnings=False,
        meta_shacl=False,
        advanced=False,
        js=False,
        debug=False)
      conforms, _, _ = r
      self.assertEqual(conforms, True)
    
    def test_reader(self):
      reader = Reader()
      g_set = GraphSet(base_iri='https://w3id.org/oc/meta/')
      g = ConjunctiveGraph()
      g.parse(source=os.path.join('oc_ocdm', 'test', 'resources', 'data_reader.json'))
      reader.import_entities_from_graph(g_set, g, resp_agent='https://orcid.org/0000-0002-8420-0696', enable_validation=True, closed=False)
      self.assertEqual(set(str(s) for s in g_set.res_to_entity.keys()), {
         'https://w3id.org/oc/meta/br/060209',
         'https://w3id.org/oc/meta/br/060182', 
         'https://w3id.org/oc/meta/ar/06034124', 
         'https://w3id.org/oc/meta/ra/06099', 
         'https://w3id.org/oc/meta/ar/060128', 
         'https://w3id.org/oc/meta/id/060313', 
         'https://w3id.org/oc/meta/ra/06098', 
         'https://w3id.org/oc/meta/re/06011'})

    def test_reader_invalid(self):
      reader = Reader()
      g_set = GraphSet(base_iri='https://w3id.org/oc/meta/')
      g = ConjunctiveGraph()
      g.parse(source=os.path.join('oc_ocdm', 'test', 'resources', 'data_reader_invalid.json'))
      reader.import_entities_from_graph(g_set, g, resp_agent='https://orcid.org/0000-0002-8420-0696', enable_validation=True, closed=False)
      self.assertEqual(set(str(s) for s in g_set.res_to_entity.keys()), {
         'https://w3id.org/oc/meta/br/060182', 
         'https://w3id.org/oc/meta/ar/06034124', 
         'https://w3id.org/oc/meta/ra/06099', 
         'https://w3id.org/oc/meta/ar/060128', 
         'https://w3id.org/oc/meta/id/060313', 
         'https://w3id.org/oc/meta/ra/06098', 
         'https://w3id.org/oc/meta/re/06011'})


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['oc_ocdm',
 'oc_ocdm.counter_handler',
 'oc_ocdm.graph',
 'oc_ocdm.graph.entities',
 'oc_ocdm.graph.entities.bibliographic',
 'oc_ocdm.metadata',
 'oc_ocdm.metadata.entities',
 'oc_ocdm.prov',
 'oc_ocdm.prov.entities',
 'oc_ocdm.resources',
 'oc_ocdm.support',
 'oc_ocdm.test',
 'oc_ocdm.test.counter_handler',
 'oc_ocdm.test.graph',
 'oc_ocdm.test.graph.entities',
 'oc_ocdm.test.graph.entities.bibliographic',
 'oc_ocdm.test.metadata',
 'oc_ocdm.test.metadata.entities',
 'oc_ocdm.test.prov',
 'oc_ocdm.test.prov.entities',
 'oc_ocdm.test.reader',
 'oc_ocdm.test.resources',
 'oc_ocdm.test.storer',
 'oc_ocdm.test.support']

package_data = \
{'': ['*'], 'oc_ocdm.test': ['coverage/*']}

install_requires = \
['SPARQLWrapper==2.0.0',
 'filelock>=3.6.0,<4.0.0',
 'pyshacl>=0.20.0,<0.21.0',
 'rdflib>=6.0.0,<7.0.0']

entry_points = \
{'console_scripts': ['test = scripts:main']}

setup_kwargs = {
    'name': 'oc-ocdm',
    'version': '7.1.13',
    'description': 'Object mapping library for manipulating RDF graphs that are compliant with the OpenCitations datamodel.',
    'long_description': '# oc_ocdm\n[<img src="https://img.shields.io/badge/powered%20by-OpenCitations-%239931FC?labelColor=2D22DE" />](http://opencitations.net)\n[![Run tests](https://github.com/opencitations/oc_ocdm/actions/workflows/run_tests.yml/badge.svg)](https://github.com/opencitations/oc_ocdm/actions/workflows/run_tests.yml)\n![Coverage](https://raw.githubusercontent.com/opencitations/oc_ocdm/master/oc_ocdm/test/coverage/coverage.svg)\n[![Documentation Status](https://readthedocs.org/projects/oc-ocdm/badge/?version=latest)](https://oc-ocdm.readthedocs.io/en/latest/?badge=latest)\n[![PyPI version](https://badge.fury.io/py/oc-ocdm.svg)](https://badge.fury.io/py/oc-ocdm)\n![PyPI](https://img.shields.io/pypi/pyversions/oc_meta)\n\n[![DOI](https://zenodo.org/badge/322327342.svg)](https://zenodo.org/badge/latestdoi/322327342)\n[![License: ISC](https://img.shields.io/badge/License-ISC-blue.svg)](https://opensource.org/licenses/ISC)\n\nDocumentation can be found here: [https://oc-ocdm.readthedocs.io](https://oc-ocdm.readthedocs.io).\n\n**oc_ocdm** is a Python &ge;3.7 library that enables the user to import, produce, modify and export RDF data\nstructures which are compliant with the [OCDM v2.0.1](https://figshare.com/articles/Metadata_for_the_OpenCitations_Corpus/3443876) specification.\n\n## User\'s guide\nThis package can be simply installed with **pip**:\n``` bash\n    pip install oc_ocdm\n```\n**Please, have a look at the notebooks [available here](https://github.com/opencitations/oc_ocdm/blob/master/notebooks/).**\n\n## Developer\'s guide\n\n### First steps\n  1. Install Poetry:\n``` bash\n    pip install poetry\n```\n  2. Clone this repository:\n``` bash\n    git clone https://github.com/iosonopersia/oc_ocdm\n    cd ./oc_ocdm\n```\n  3. Install all the dependencies:\n``` bash\n    poetry install\n```\n  4. Build the package (_output dir:_ `dist`):\n``` bash\n    poetry build\n```\n  5. Globally install the package (_alternatively, you can also install it inside a virtual-env,\n  by providing the full path to the .tar.gz archive_):\n``` bash\n    pip install ./dist/oc_ocdm-<VERSION>.tar.gz\n```\n  6. If everything went the right way, than you should be able to use the `oc_ocdm` library in your Python modules as follows:\n``` python\n    from oc_ocdm.graph import GraphSet\n    from oc_ocdm.graph.entities.bibliographic import AgentRole\n    # ...\n```\n\n### How to run the tests\nJust run the following command inside the root project folder:\n``` bash\n    poetry run test\n```\n\n### How to manage the project using Poetry\nSee [Poetry commands documentation](https://python-poetry.org/docs/cli/).\n\n**AAA: when adding a non-dev dependency via `poetry add`, always remember to add\nthat same dependency to the `autodoc_mock_imports` list in `docs/source/conf.py`**\n(otherwise "Read the Docs" won\'t be able to compile the documentation correctly!).\n\n### How to publish the package onto Pypi\n``` bash\n    poetry publish --build\n```\n### Install dependencies needed for the documentation\n``` bash\n    pip install Sphinx sphinx_rtd_theme\n```\n### How to generate the documentation\n``` bash\n    rm ./docs/source/modules/*\n    sphinx-apidoc  -o ./docs/source/modules oc_ocdm *test*\n```\n\n### How to build the documentation\n___\n**Warning! In order to avoid getting the following `WARNING: html_static_path entry \'_static\' does not exist`, you\'ll\nneed to manually create an empty `_static` folder with the command:**\n``` bash\n    mkdir docs/source/_static\n```\n___\n  1. Always remember to move inside the `docs` folder:\n``` bash\n    cd docs\n```\n  2. Use the Makefile provided to build the docs:\n      + _on Windows_\n        ```\n            make.bat html\n        ```\n      + _on Linux and MacOs_\n        ```\n            make html\n        ```\n  3. Open the `build/html/index.html` file with a web browser of your choice!\n\n## Acknowledgements\nThis work has been funded by the project “Open Biomedical Citations in Context Corpus”\n(Wellcome Trust, Grant n. 214471/Z/18/Z) and the project “Wikipedia Citations in Wikidata”\n(Wikimedia Foundation, https://meta.wikimedia.org/wiki/Wikicite/grant/Wikipedia_Citations_in_Wikidata).\n\nWe would like to thank (in alphabetic order) Fabio Mariani (@FabioMariani), Arcangelo\nMassari (@arcangelo7), and Gabriele Pisciotta (@GabrielePisciotta) for the constructive feedback.\n',
    'author': 'Silvio Peroni',
    'author_email': 'essepuntato@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://opencitations.net',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.4,<4.0.0',
}


setup(**setup_kwargs)



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
import json
import os
import unittest
from platform import system
from shutil import rmtree
from zipfile import ZipFile

from rdflib import ConjunctiveGraph, URIRef, compare

from oc_ocdm.graph.graph_set import GraphSet
from oc_ocdm.prov.prov_set import ProvSet
from oc_ocdm.storer import Storer


class TestStorer(unittest.TestCase):
    def setUp(self):
        self.resp_agent = "http://resp_agent.test/"
        self.base_iri = "http://test/"
        self.graph_set = GraphSet(self.base_iri, "", "060", False)
        self.prov_set = ProvSet(self.graph_set, self.base_iri, "", False)
        self.br = self.graph_set.add_br(self.resp_agent)

    def tearDown(self):
        rmtree(os.path.join("oc_ocdm", "test", "storer", "data"))

    def test_store_graphs_in_file(self):
        base_dir = os.path.join("oc_ocdm", "test", "storer", "data", "rdf") + os.sep
        is_unix = system() != "Windows"
        with self.subTest("output_format=json-ld, zip_output=True"):
            storer = Storer(self.graph_set, context_map={}, dir_split=10000, n_file_item=1000, default_dir="_", output_format='json-ld', zip_output=True)
            self.prov_set.generate_provenance()
            prov_storer = Storer(self.prov_set, context_map={}, dir_split=10000, n_file_item=1000, default_dir="_", output_format='json-ld', zip_output=True)
            storer.store_all(base_dir, self.base_iri)
            prov_storer.store_all(base_dir, self.base_iri)
            self.graph_set.commit_changes()
            with ZipFile(os.path.join(base_dir, "br", "060", "10000", "1000.zip"), mode="r") as archive:
                with archive.open("1000.json") as f:
                    data = json.load(f)
                    self.assertEqual(data, [{'@graph': [{'@id': 'http://test/br/0601', '@type': ['http://purl.org/spar/fabio/Expression']}], '@id': 'http://test/br/'}])
            with ZipFile(os.path.join(base_dir, "br", "060", "10000", "1000", "prov", "se.zip"), mode="r") as archive:
                with archive.open("se.json") as f:
                    data = [{g:[{k:v for k,v in datum.items() if k != "http://www.w3.org/ns/prov#generatedAtTime"} for datum in data] if g == "@graph" else data for g, data in graph.items()} for graph in json.load(f)]
                    self.assertEqual(data, [{'@graph': [{
                        '@id': 'http://test/br/0601/prov/se/1', 
                        '@type': ['http://www.w3.org/ns/prov#Entity'], 
                        'http://purl.org/dc/terms/description': [{'@value': "The entity 'http://test/br/0601' has been created."}], 
                        'http://www.w3.org/ns/prov#specializationOf': [{'@id': 'http://test/br/0601'}], 
                        'http://www.w3.org/ns/prov#wasAttributedTo': [{'@id': 'http://resp_agent.test/'}]}], '@id': 'http://test/br/0601/prov/'}])
            if is_unix:
                self.assertTrue(os.path.exists(os.path.join(base_dir, "br", "060", "10000", "1000.zip.lock")))
                self.assertTrue(os.path.exists(os.path.join(base_dir, "br", "060", "10000", "1000", "prov", "se.zip.lock")))
        with self.subTest("output_format=json-ld, zip_output=False"):
            base_dir_1 = os.path.join("oc_ocdm", "test", "storer", "data", "rdf_1") + os.sep
            storer = Storer(self.graph_set, context_map={}, dir_split=10000, n_file_item=1000, default_dir="_", output_format='json-ld', zip_output=False)
            self.prov_set.generate_provenance()
            prov_storer = Storer(self.prov_set, context_map={}, dir_split=10000, n_file_item=1000, default_dir="_", output_format='json-ld', zip_output=False)
            storer.store_all(base_dir_1, self.base_iri)
            prov_storer.store_all(base_dir_1, self.base_iri)
            self.graph_set.commit_changes()
            with open(os.path.join(base_dir_1, "br", "060", "10000", "1000.json")) as f:
                data = json.load(f)
                self.assertEqual(data, [{'@graph': [{'@id': 'http://test/br/0601', '@type': ['http://purl.org/spar/fabio/Expression']}], '@id': 'http://test/br/'}])
            with open(os.path.join(base_dir_1, "br", "060", "10000", "1000", "prov", "se.json")) as f:
                data = [{g:[{k:v for k,v in datum.items() if k != "http://www.w3.org/ns/prov#generatedAtTime"} for datum in data] if g == "@graph" else data for g, data in graph.items()} for graph in json.load(f)]
                self.assertEqual(data, [{'@graph': [{
                    '@id': 'http://test/br/0601/prov/se/1', 
                    '@type': ['http://www.w3.org/ns/prov#Entity'], 
                    'http://purl.org/dc/terms/description': [{'@value': "The entity 'http://test/br/0601' has been created."}], 
                    'http://www.w3.org/ns/prov#specializationOf': [{'@id': 'http://test/br/0601'}], 
                    'http://www.w3.org/ns/prov#wasAttributedTo': [{'@id': 'http://resp_agent.test/'}]}], '@id': 'http://test/br/0601/prov/'}])
            if is_unix:
                self.assertTrue(os.path.exists(os.path.join(base_dir_1, "br", "060", "10000", "1000.json.lock")))
                self.assertTrue(os.path.exists(os.path.join(base_dir_1, "br", "060", "10000", "1000", "prov", "se.json.lock")))
        with self.subTest("output_format=nquads, zip_output=True"):
            base_dir_2 = os.path.join("oc_ocdm", "test", "storer", "data", "rdf_2") + os.sep
            storer = Storer(self.graph_set, context_map={}, dir_split=10000, n_file_item=1000, default_dir="_", output_format='nquads', zip_output=True)
            self.prov_set.generate_provenance()
            prov_storer = Storer(self.prov_set, context_map={}, dir_split=10000, n_file_item=1000, default_dir="_", output_format='nquads', zip_output=True)
            storer.store_all(base_dir_2, self.base_iri)
            prov_storer.store_all(base_dir_2, self.base_iri)
            self.graph_set.commit_changes()
            with ZipFile(os.path.join(base_dir_2, "br", "060", "10000", "1000.zip"), mode="r") as archive:
                with archive.open("1000.nt") as f:
                    data = f.read().decode("utf-8")
                    self.assertEqual(data, "<http://test/br/0601> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://purl.org/spar/fabio/Expression> <http://test/br/> .\n\n")
            with ZipFile(os.path.join(base_dir_2, "br", "060", "10000", "1000", "prov", "se.zip"), mode="r") as archive:
                with archive.open("se.nq") as f:
                    data = f.read().decode("utf-8")
                    data_g = ConjunctiveGraph()
                    expected_data_g = ConjunctiveGraph()
                    data_g.parse(data=data, format="nquads")
                    expected_data_g.parse(data="""
                        <http://test/br/0601/prov/se/1> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.w3.org/ns/prov#Entity> <http://test/br/0601/prov/> .
                        <http://test/br/0601/prov/se/1> <http://www.w3.org/ns/prov#specializationOf> <http://test/br/0601> <http://test/br/0601/prov/> .
                        <http://test/br/0601/prov/se/1> <http://www.w3.org/ns/prov#wasAttributedTo> <http://resp_agent.test/> <http://test/br/0601/prov/> .
                        <http://test/br/0601/prov/se/1> <http://purl.org/dc/terms/description> "The entity 'http://test/br/0601' has been created." <http://test/br/0601/prov/> .
                    """, format="nquads")
                    for s, p, o, c in data_g.quads():
                        if p == URIRef("http://www.w3.org/ns/prov#generatedAtTime"):
                            data_g.remove((s, p, o, c))
                    self.assertTrue(compare.isomorphic(data_g, expected_data_g))
            if is_unix:
                self.assertTrue(os.path.exists(os.path.join(base_dir_2, "br", "060", "10000", "1000.zip.lock")))
                self.assertTrue(os.path.exists(os.path.join(base_dir_2, "br", "060", "10000", "1000", "prov", "se.zip.lock")))
        with self.subTest("output_format=nquads, zip_output=False"):
            base_dir_3 = os.path.join("oc_ocdm", "test", "storer", "data", "rdf_3") + os.sep
            storer = Storer(self.graph_set, context_map={}, dir_split=10000, n_file_item=1000, default_dir="_", output_format='nquads', zip_output=False)
            self.prov_set.generate_provenance()
            prov_storer = Storer(self.prov_set, context_map={}, dir_split=10000, n_file_item=1000, default_dir="_", output_format='nquads', zip_output=False)
            storer.store_all(base_dir_3, self.base_iri)
            prov_storer.store_all(base_dir_3, self.base_iri)
            self.graph_set.commit_changes()
            prov_unzipped = ConjunctiveGraph()
            expected_prov_unzipped = ConjunctiveGraph()
            with open(os.path.join(base_dir_3, "br", "060", "10000", "1000.nt"), "r", encoding="utf-8") as f:
                data_unzipped = f.read()
            prov_unzipped.parse(source=os.path.join(base_dir_3, "br", "060", "10000", "1000", "prov", "se.nq"), format="nquads")
            expected_prov_unzipped.parse(data="""
                <http://test/br/0601/prov/se/1> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.w3.org/ns/prov#Entity> <http://test/br/0601/prov/> .
                <http://test/br/0601/prov/se/1> <http://www.w3.org/ns/prov#specializationOf> <http://test/br/0601> <http://test/br/0601/prov/> .
                <http://test/br/0601/prov/se/1> <http://www.w3.org/ns/prov#wasAttributedTo> <http://resp_agent.test/> <http://test/br/0601/prov/> .
                <http://test/br/0601/prov/se/1> <http://purl.org/dc/terms/description> "The entity 'http://test/br/0601' has been created." <http://test/br/0601/prov/> .
            """, format="nquads")
            for s, p, o, c in prov_unzipped.quads():
                if p == URIRef("http://www.w3.org/ns/prov#generatedAtTime"):
                    prov_unzipped.remove((s, p, o, c))
            self.assertEqual(data_unzipped, "<http://test/br/0601> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://purl.org/spar/fabio/Expression> <http://test/br/> .\n\n")
            self.assertTrue(compare.isomorphic(prov_unzipped, expected_prov_unzipped))
            if is_unix:
                self.assertTrue(os.path.exists(os.path.join(base_dir_3, "br", "060", "10000", "1000.nt.lock")))
                self.assertTrue(os.path.exists(os.path.join(base_dir_3, "br", "060", "10000", "1000", "prov", "se.nq.lock")))


if __name__ == '__main__':
    unittest.main()